/*
 * Copyright (c) 2001 Edouard TISSERANT <tissered@esstin.u-nancy.fr>
 *  Parts inspired from Shane Watts <shane@bofh.asn.au> Xfree 3 Acecad Driver
 *  Thanks to Emily, from AceCad, For giving me documents.
 * Copyright (c) 2010 Gerard Krol <gerard_@gna.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 *
 */

#ifndef _WIZARDPEN_H_
#define _WIZARDPEN_H_

/******************************************************************************
 *      Definitions
 *      structs, typedefs, #defines, enums
 *****************************************************************************/
#define WIZARDPEN_PACKET_SIZE   7

#define WIZARDPEN_CONFIG        "a"     /* Send configuration (max coords) */

#define WIZARDPEN_ABSOLUTE      'F'     /* Absolute mode */
#define WIZARDPEN_RELATIVE      'E'     /* Relative mode */

#define WIZARDPEN_UPPER_ORIGIN  "b"     /* Origin upper left */

#define WIZARDPEN_RESET         "z0"    /* Reset the tablet */
#define WIZARDPEN_PROMPT_MODE   "B"     /* Prompt mode */
#define WIZARDPEN_STREAM_MODE   "@"     /* Stream mode */
#define WIZARDPEN_INCREMENT     'I'     /* Set increment */
#define WIZARDPEN_BINARY_FMT    "zb"    /* Binary reporting */

#define WIZARDPEN_PROMPT        "P"     /* Prompt for current position */

#define WIZARDPEN_BTN_TOUCH     1       /* Used for stylus button events */
#define WIZARDPEN_BTN_STYLUS    2
#define WIZARDPEN_BTN_STYLUS2   4
#define WIZARDPEN_BTN_SIDE      8

#define PHASING_BIT     0x80
#define PROXIMITY_BIT   0x40
#define TABID_BIT       0x20
#define XSIGN_BIT       0x10
#define YSIGN_BIT       0x08
#define BUTTON_BITS     0x07
#define COORD_BITS      0x7f

#define ABSOLUTE_FLAG   1
#define USB_FLAG        2
#define AUTODEV_FLAG    4
#define AVAIL_FLAG      8

#define NOTAVAIL ((errno == ENODEV) || (errno == ENXIO) || (errno == ENOENT))

#define milisleep(ms) xf86usleep (ms * 1000)

#define SYSCALL(call) while(((call) == -1) && (errno == EINTR))

static const char * wizardpen_initstr = WIZARDPEN_BINARY_FMT WIZARDPEN_STREAM_MODE;

typedef struct
{
    // settings from xorg.conf/hal
    // see README-XOrgConfig for descriptions
    int topZ;
    int bottomZ;
    int topX;
    int topY;
    int bottomX;
    int bottomY;
    int screenX;
    int screenY;
    int debugLevel;
    int rotate90;
    int mouseSpeed;
    int mouseAccel;
    BOOL tpcButton;
    BOOL keepShape;
    BOOL reportScreen;

    // local variables
    XISBuffer *buffer;    // buffer for reading from the tablet
    int increment;        // increment between transmits
    int oldX;             // previous X position
    int oldY;             // previous Y position
    int oldZ;             // previous Z position
    int oldProximity;     // previous proximity
    int oldButtons;       // previous buttons state
    int oldReportedButtons; // previous reported buttons state
    int maxX;             // max X value
    int maxY;             // max Y value
    int maxZ;             // max Y value
    float aspectRatio;    // aspect ratio (width/height) of the tablet
    char reportSpeed;     // report speed
    int flags;            // various flags
    int packeti;          // number of bytes read
    int PacketSize;       // number of bytes read
    unsigned char packet[WIZARDPEN_PACKET_SIZE];  // data read on the device
    int activeButtons;    // bitfield that contains button state
} WizardPenPrivateRec, *WizardPenPrivatePtr;


/******************************************************************************
 *      Declarations
 *****************************************************************************/

#ifdef XFree86LOADER
    static MODULESETUPPROTO( SetupProc );
    static void TearDownProc (void *);
#endif

static Bool DeviceControl (DeviceIntPtr, int);
static Bool DeviceOn (DeviceIntPtr);
static Bool DeviceOff (DeviceIntPtr);
static Bool DeviceClose (DeviceIntPtr);
static Bool DeviceInit (DeviceIntPtr);
static Bool QueryHardware (WizardPenPrivatePtr);
static void NewPacket (WizardPenPrivatePtr priv);
static Bool WizardPenGetPacket (WizardPenPrivatePtr);
#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 12
static InputInfoPtr WizardPenPreInit(InputDriverPtr, IDevPtr , int);
#else
static int WizardPenPreInit(InputDriverPtr, InputInfoPtr, int);
#endif

#ifdef LINUX_INPUT
    static void USBReadInput (InputInfoPtr);
    static Bool USBQueryHardware (InputInfoPtr);
    static int IsUSBLine(int);
    static Bool WizardPenAutoDevProbe(InputInfoPtr, int);
#endif

#endif
